import com.ibm.mq.*; // Include the WebSphere MQ classes for Java package

import org.apache.log4j.Logger;
import com.ibm.mq.pcf.CMQC;
import java.util.*;
import java.io.*;

/**
* EventMonitor is a Java program that acts as an event monitor - reads an  
* event queue and tells you if anything appears on it.         
*                                                              
* It's second parameter is the queue manager name, the first is
* the event queue name. If these are not supplied it uses the  
* defaults.  
* @author      Created by Sam Garforth
* Created on 03-Nov-2005.
* @version     $Name:  $ $Revision: 1.0 $ $Date: 2005/11/03 15:50:36 $
* @since       1.0.
**/
public class EventMonitor 
{
  /** The queue to be browsed. By default this is SYSTEM.ADMIN.PERFM.EVENT */
  private String queueName;
  /** The queue manager containing the queue to be browsed. By default this is the default local queue manager */ 
  private String queueManagerName;
  /** Interface/handle to the log4j error logger.  */
  private Logger logger;
  /**
   * The command line entry point. Usage: EventMonitor [EventQueue] [QMgrName]
   * @param eventQueue The name of the event queue to be read.
   * @param queueManagerName The name of the queue manager holding the event queue.
   */
  public static void main(String[] args)
  {
	  String firstParam = "SYSTEM.ADMIN.PERFM.EVENT";
	  String secondParam = "";
	  if (args.length > 0)
	  {
		if (("-?".compareToIgnoreCase(args[0]) == 0) || 
		("?".compareToIgnoreCase(args[0]) == 0))
		{
		  System.out.println("Usage: EventMonitor [EventQueue] [QMgrName]\n");
		  return;
	    }
		firstParam = args[0];
		if (args.length > 1)
		  secondParam = args[1];
	  }
    EventMonitor eventMonitor = new EventMonitor(firstParam,secondParam);
    eventMonitor.execute();
  }
  /**
   * Constructor gets a connection to the logger and sets the queue to be read.
   */
  public EventMonitor(String queueName, String queueManagerName)
  {
    logger = Logger.getLogger(this.getClass());
    setQueueName(queueName);
    setQueueManagerName(queueManagerName);
  }
  /**
   * Sets the name of the queue to be browsed for messages.
   * @param queueName The name of the queue which contains the messages to be browsed.
   */
  public void setQueueName(String queueName)
  {
    this.queueName = queueName;
  }
  /**
   * Sets the name of the queue manager containing the messages to be browsed.
   * @param queueManagerName The name of the queue manager which contains the messages to be browsed.
   */
  public void setQueueManagerName(String queueManagerName)
  {
    this.queueManagerName = queueManagerName;
  }
  /**
   * Browses the specified queue and lists the messages.
   */
  public void execute()
  {
    MQMessage retrievedMessage; // a browsed message
    MQQueueManager qMgr;        // define a queue manager object
    MQUtilities mqUtils = new MQUtilities();
    String stdout = "";
    try
    {
      // This next part isn't nice but seems the only way to stop the IBM code 
      // printing out unwanted messages
      PipedInputStream pin=new PipedInputStream();
      PipedOutputStream pout=new PipedOutputStream(pin);
	  System.setErr(new PrintStream(pout,true)); 
	  
      logger.debug("about to connect to the qmgr ");
      qMgr = new MQQueueManager(queueManagerName);
      int openOptions =
        MQC.MQOO_INPUT_AS_Q_DEF
          | MQC.MQOO_INQUIRE
          | MQC.MQOO_BROWSE
          | MQC.MQOO_FAIL_IF_QUIESCING;
      MQQueue queue = qMgr.accessQueue(queueName, openOptions);
      /**
       * loop forever or until there is an exception. We used to have this as 
       * unlimited wait because if it's done from a client and the client dies  
       * then the server doesn't notice and keeps the queue open      
       * so now we loop on a 5 second wait 
       */
      try {
      while (1 == 1)
      {
      MQGetMessageOptions gmo = new MQGetMessageOptions();
      // accept the defaults
      // same as  MQGMO_DEFAULT
      gmo.waitInterval = 5000;
      gmo.options = gmo.options | MQC.MQGMO_WAIT | MQC.MQGMO_BROWSE_NEXT;
      ArrayList messageCollection = new ArrayList();
      boolean msgsavail = true;
      while (msgsavail)
      {
        retrievedMessage = new MQMessage();
        try
        {
          queue.get(retrievedMessage, gmo);
          messageCollection.add(retrievedMessage);
          
        }
        catch (MQException ex)
        {
          msgsavail = false;
          if (ex.reasonCode != CMQC.MQRC_NO_MSG_AVAILABLE)
            throw ex;
        }
      }
      if (!messageCollection.isEmpty())
      {
      stdout = (new MQUtilities()).xmlFromPCFMessageCollection(messageCollection);
      logger.debug(stdout);
      System.out.println(stdout);
      }
    }}
      finally
      {
      queue.close();
      qMgr.disconnect();
      }
    }
    catch (MQException ex)
    {
      logger.error(
        "A WebSphere MQ error occurred : Reason code "
          + ex.reasonCode
          + " "
          + mqUtils.getReasonName(ex.reasonCode),
        ex);
    }
    catch (java.io.IOException ex)
    {
      logger.error(
        "An error occurred whilst writing to the message buffer: " + ex,
        ex);
    }
    logger.debug("PCF XML: " + stdout);
  }
}
